@@ -42,6 +42,8 @@ typedef struct
   gint y_root;
   cairo_rectangle_int_t rectangle;
   cairo_rectangle_int_t rectangle_root;
+  GtkWidget *size_window;
+  GtkWidget *size_label;
 } RubberBandData;
 
 /* For non-composited environments */
@@ -265,6 +267,10 @@ fallback:
 #endif
   TRACE ("Get the mouse cursor and its image through fallback mode");
 
+  /* cursors are not scaled */
+  if (gdk_window_get_scale_factor (root) != 1)
+    return NULL;
+
   cursor = gdk_cursor_new_for_display (display, GDK_LEFT_PTR);
   cursor_pixbuf = gdk_cursor_get_image (cursor);
 
@@ -298,6 +304,7 @@ static GdkPixbuf
   GdkPixbuf *screenshot;
   GdkWindow *root;
 
+  gint scale;
   GdkRectangle rectangle;
 
   /* Get the root window */
@@ -312,6 +319,8 @@ static GdkPixbuf
     		                                           find_wm_window (xwindow));
     }
 
+  scale = gdk_window_get_scale_factor (window);
+
   rectangle.width = gdk_window_get_width (window);
   rectangle.height = gdk_window_get_height (window);
   gdk_window_get_origin (window, &rectangle.x, &rectangle.y);
@@ -343,7 +352,7 @@ static GdkPixbuf
   if (y_orig + height > gdk_screen_height ())
     height = gdk_screen_height () - y_orig;
 
-  /* Take the screenshot from the root GdkWindow, to grab things such as
+  /* Capture the screenshot from the root GdkWindow, to grab things such as
    * menus. */
 
   TRACE ("Grab the screenshot");
@@ -372,7 +381,7 @@ static GdkPixbuf
           gboolean has_alpha = gdk_pixbuf_get_has_alpha (screenshot);
 
           tmp =
-            gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, width, height);
+            gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, width * scale, height * scale);
           gdk_pixbuf_fill (tmp, 0);
 
           for (i = 0; i < rectangle_count; i++)
@@ -412,11 +421,11 @@ static GdkPixbuf
                   rec_y = 0;
                 }
 
-              if (x_orig + rec_x + rec_width > gdk_screen_width ())
-                rec_width = gdk_screen_width () - x_orig - rec_x;
+              if (x_orig + rec_x + rec_width > gdk_screen_width () * scale)
+                rec_width = gdk_screen_width () * scale - x_orig - rec_x;
 
-              if (y_orig + rec_y + rec_height > gdk_screen_height ())
-                rec_height = gdk_screen_height () - y_orig - rec_y;
+              if (y_orig + rec_y + rec_height > gdk_screen_height () * scale)
+                rec_height = gdk_screen_height () * scale - y_orig - rec_y;
 
               for (y = rec_y; y < rec_y + rec_height; y++)
                 {
@@ -463,10 +472,10 @@ static GdkPixbuf
             GdkRectangle rectangle_window, rectangle_cursor;
 
             /* rectangle_window stores the window coordinates */
-            rectangle_window.x = x_orig;
-            rectangle_window.y = y_orig;
-            rectangle_window.width = width;
-            rectangle_window.height = height;
+            rectangle_window.x = x_orig * scale;
+            rectangle_window.y = y_orig * scale;
+            rectangle_window.width = width * scale;
+            rectangle_window.height = height * scale;
 
             /* rectangle_cursor stores the cursor coordinates */
             rectangle_cursor.x = cursorx;
@@ -484,12 +493,12 @@ static GdkPixbuf
                 TRACE ("Compose the two pixbufs");
 
                 gdk_pixbuf_composite (cursor_pixbuf, screenshot,
-                                      cursorx - x_orig -xhot,
-                                      cursory - y_orig -yhot,
+                                      cursorx - rectangle_window.x - xhot,
+                                      cursory - rectangle_window.y - yhot,
                                       rectangle_cursor.width,
                                       rectangle_cursor.height,
-                                      cursorx - x_orig - xhot,
-                                      cursory - y_orig -yhot,
+                                      cursorx - rectangle_window.x - xhot,
+                                      cursory - rectangle_window.y - yhot,
                                       1.0, 1.0,
                                       GDK_INTERP_BILINEAR,
                                       255);
@@ -642,6 +651,7 @@ static gboolean cb_button_released (GtkWidget *widget,
       if (rbdata->rubber_banding)
         {
           gtk_dialog_response (GTK_DIALOG (widget), GTK_RESPONSE_NONE);
+          gtk_widget_destroy (rbdata->size_window);
           return TRUE;
         }
       else
@@ -653,10 +663,99 @@ static gboolean cb_button_released (GtkWidget *widget,
 
 
 
+
+static gboolean cb_size_window_draw (GtkWidget *widget,
+                                     cairo_t *cr,
+                                     gpointer user_data)
+{
+  cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
+  cairo_set_source_rgba (cr, 1.0, 1.0, 1.0, 0.0);
+  cairo_paint (cr);
+
+  return FALSE;
+}
+
+
+
+static void size_window_get_offset (GtkWidget *widget,
+                                    gint digits,
+                                    gint x_event,
+                                    gint y_event,
+                                    gint *x_offset,
+                                    gint *y_offset)
+{
+  GdkRectangle geometry;
+  gint relative_x, relative_y;
+
+  GdkDisplay *display = gtk_widget_get_display (widget);
+  GdkMonitor *monitor = gdk_display_get_monitor_at_point (display, x_event, y_event);
+  gdk_monitor_get_geometry (monitor, &geometry);
+
+  relative_x = x_event - geometry.x;
+  relative_y = y_event - geometry.y;
+
+  *x_offset = -2;
+  *y_offset = -4;
+
+  if (relative_x > geometry.width - (digits * 9))
+    *x_offset -= digits * 9;
+
+  if (relative_y > geometry.height - 20)
+    *y_offset -= 20;
+}
+
+
+
+static void create_size_window (RubberBandData *rbdata)
+{
+  GtkWidget *window, *label;
+  GtkCssProvider *css_provider;
+  GtkStyleContext *context;
+  GdkVisual *visual;
+
+  rbdata->size_window = window = gtk_window_new (GTK_WINDOW_POPUP);
+  gtk_container_set_border_width (GTK_CONTAINER (window), 0);
+  gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
+  gtk_window_set_default_size (GTK_WINDOW (window), 100, 50);
+  gtk_widget_set_size_request (GTK_WIDGET (window), 100, 50);
+  gtk_window_set_decorated (GTK_WINDOW (window), FALSE);
+  gtk_widget_set_app_paintable (GTK_WIDGET (window), TRUE);
+  gtk_window_set_skip_taskbar_hint (GTK_WINDOW (window), FALSE);
+  g_signal_connect (G_OBJECT (window), "draw",
+                    G_CALLBACK (cb_size_window_draw), NULL);
+
+  visual = gdk_screen_get_rgba_visual (gtk_widget_get_screen (window));
+  gtk_widget_set_visual (window, visual);
+
+  rbdata->size_label = label = gtk_label_new ("");
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+  gtk_widget_set_valign (label, GTK_ALIGN_START);
+  gtk_widget_set_margin_start (label, 6);
+  gtk_widget_set_margin_top (label, 6);
+  gtk_container_add (GTK_CONTAINER (window), label);
+
+  css_provider = gtk_css_provider_new ();
+  gtk_css_provider_load_from_data (css_provider,
+    "label { font-family: monospace; color: #fff; text-shadow: 1px 1px 0px black; }", -1, NULL);
+
+  context = gtk_widget_get_style_context (GTK_WIDGET (label));
+  gtk_style_context_add_provider (context,
+                                  GTK_STYLE_PROVIDER (css_provider),
+                                  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  g_object_unref (css_provider);
+
+  gtk_widget_show_all (GTK_WIDGET (window));
+}
+
+
+
 static gboolean cb_motion_notify (GtkWidget *widget,
                                   GdkEventMotion *event,
                                   RubberBandData *rbdata)
 {
+  gchar *coords;
+  gint rect_width, rect_height, x_offset, y_offset;
+
   if (rbdata->left_pressed)
     {
       cairo_rectangle_int_t *new_rect, *new_rect_root;
@@ -683,6 +782,26 @@ static gboolean cb_motion_notify (GtkWidget *widget,
           old_rect.y = new_rect->y;
           old_rect.width = new_rect->width;
           old_rect.height = new_rect->height;
+
+          rect_width = old_rect.width;
+          rect_height = old_rect.height;
+
+          /* Take into account when the rectangle is moved out of screen */
+          if (old_rect.x < 0) rect_width += old_rect.x;
+          if (old_rect.y < 0) rect_height += old_rect.y;
+
+          coords = g_strdup_printf ("%d x %d", rect_width, rect_height);
+
+          size_window_get_offset (rbdata->size_window, strlen (coords),
+                                  event->x_root, event->y_root,
+                                  &x_offset, &y_offset);
+
+          gtk_window_move (GTK_WINDOW (rbdata->size_window),
+                           event->x_root + x_offset,
+                           event->y_root + y_offset);
+
+          gtk_label_set_text (GTK_LABEL (rbdata->size_label), coords);
+          g_free (coords);
         }
 
       if (rbdata->move_rectangle)
@@ -804,11 +923,10 @@ static GdkPixbuf
   GtkWidget *window;
   RubberBandData rbdata;
   GdkPixbuf *screenshot = NULL;
-  GdkWindow *root;
-  GdkDevice *pointer, *keyboard;
   GdkGrabStatus res;
   GdkSeat   *seat;
   GdkCursor *xhair_cursor;
+  GdkDisplay *display;
 
   /* Initialize the rubber band data */
   rbdata.left_pressed = FALSE;
@@ -849,9 +967,9 @@ static GdkPixbuf
 
   /* This window is not managed by the window manager, we have to set everything
    * ourselves */
+  display = gdk_display_get_default ();
   gtk_widget_realize (window);
-  xhair_cursor = gdk_cursor_new_for_display (gdk_display_get_default (), GDK_CROSSHAIR);
-  gdk_window_set_cursor (gtk_widget_get_window (window), xhair_cursor);
+  xhair_cursor = gdk_cursor_new_for_display (display, GDK_CROSSHAIR);
   gdk_window_set_override_redirect (gtk_widget_get_window (window), TRUE);
   gtk_widget_set_size_request (window,
                                gdk_screen_get_width (gdk_screen_get_default ()),
@@ -859,7 +977,7 @@ static GdkPixbuf
   gdk_window_raise (gtk_widget_get_window (window));
   gtk_widget_show_now (window);
   gtk_widget_grab_focus (window);
-  gdk_flush ();
+  gdk_display_flush (display);
 
   /* Wait 100ms before grabbing devices, useful when invoked by global hotkey
    * because xfsettings will grab the key for a moment */ 
@@ -867,51 +985,32 @@ static GdkPixbuf
 
   /* Grab the mouse and the keyboard to prevent any interaction with other
    * applications */
-  seat = gdk_display_get_default_seat (gdk_display_get_default ());
-  pointer = gdk_seat_get_pointer (seat);
-  keyboard = gdk_seat_get_keyboard (seat);
+  seat = gdk_display_get_default_seat (display);
 
-  res = gdk_device_grab (keyboard, gtk_widget_get_window (window),
-                         GDK_OWNERSHIP_NONE, FALSE,
-                         GDK_KEY_PRESS_MASK |
-                         GDK_KEY_RELEASE_MASK,
-                         NULL, GDK_CURRENT_TIME);
+  res = gdk_seat_grab (seat, gtk_widget_get_window (window),
+                       GDK_SEAT_CAPABILITY_ALL, FALSE, xhair_cursor,
+                       NULL, NULL, NULL);
 
   if (res != GDK_GRAB_SUCCESS)
     {
       gtk_widget_destroy (window);
       g_object_unref (xhair_cursor);
-      g_warning ("Failed to grab keyboard");
+      g_warning ("Failed to grab seat");
       return NULL;
     }
 
-  res = gdk_device_grab (pointer, gtk_widget_get_window (window),
-                         GDK_OWNERSHIP_NONE, FALSE,
-                         GDK_POINTER_MOTION_MASK |
-                         GDK_BUTTON_PRESS_MASK | 
-                         GDK_BUTTON_RELEASE_MASK,
-                         NULL, GDK_CURRENT_TIME);
-
-  if (res != GDK_GRAB_SUCCESS)
-    {
-      gtk_widget_destroy (window);
-      g_object_unref (xhair_cursor);
-      gdk_device_ungrab (keyboard, GDK_CURRENT_TIME);
-      g_warning ("Failed to grab pointer");
-      return NULL;
-    }
+  /* set up the window showing the screenshot size */
+  create_size_window (&rbdata);
 
   gtk_dialog_run (GTK_DIALOG (window));
   gtk_widget_destroy (window);
   g_object_unref (xhair_cursor);
-  gdk_flush();
+  gdk_display_flush (display);
 
   if (rbdata.cancelled)
     goto cleanup;
 
   /* Grab the screenshot on the main window */
-  root = gdk_get_default_root_window ();
-
   screenshot = capture_rectangle_screenshot (rbdata.rectangle_root.x,
                                              rbdata.rectangle_root.y,
                                              rbdata.rectangle.width,
@@ -919,10 +1018,9 @@ static GdkPixbuf
                                              delay);
 
   cleanup:
-  /* Ungrab the mouse and the keyboard */
-  gdk_device_ungrab (pointer, GDK_CURRENT_TIME);
-  gdk_device_ungrab (keyboard, GDK_CURRENT_TIME);
-  gdk_flush ();
+  gtk_widget_destroy (rbdata.size_window);
+  gdk_seat_ungrab (seat);
+  gdk_display_flush (display);
 
   return screenshot;
 }
@@ -938,23 +1036,38 @@ region_filter_func (GdkXEvent *xevent, GdkEvent *event, RbData *rbdata)
   Display *display;
   Window root_window;
   int key;
+  int event_type;
+  gboolean is_xinput = FALSE;
 
   display = gdk_x11_get_default_xdisplay ();
   root_window = gdk_x11_get_default_root_xwindow ();
 
-  if (x_event->type != GenericEvent)
-    return GDK_FILTER_CONTINUE;
+  event_type = x_event->type;
 
-  switch (x_event->xgeneric.evtype)
+  if (event_type == GenericEvent)
+    {
+      event_type = x_event->xgeneric.evtype;
+      is_xinput = TRUE;
+    }
+
+  switch (event_type)
     {
       /* Start dragging the rectangle out */
-      case XI_ButtonPress:
+      case XI_ButtonPress: /* ButtonPress */
         TRACE ("Start dragging the rectangle");
 
-        device_event = (XIDeviceEvent*) x_event->xcookie.data;
-
-        rbdata->rectangle.x = rbdata->x1 = device_event->root_x;
-        rbdata->rectangle.y = rbdata->y1 = device_event->root_y;
+        if (is_xinput)
+          {
+            device_event = (XIDeviceEvent*) x_event->xcookie.data;
+            rbdata->rectangle.x = rbdata->x1 = device_event->root_x;
+            rbdata->rectangle.y = rbdata->y1 = device_event->root_y;
+          }
+        else
+          {
+            rbdata->rectangle.x = rbdata->x1 = x_event->xkey.x_root;
+            rbdata->rectangle.y = rbdata->y1 = x_event->xkey.y_root;
+          }
+        
         rbdata->rectangle.width = 0;
         rbdata->rectangle.height = 0;
         rbdata->pressed = TRUE;
@@ -965,7 +1078,7 @@ region_filter_func (GdkXEvent *xevent, GdkEvent *event, RbData *rbdata)
       break;
 
       /* Finish dragging the rectangle out */
-      case XI_ButtonRelease:
+      case XI_ButtonRelease: /* ButtonRelease */
         if (rbdata->pressed)
           {
             if (rbdata->rectangle.width > 0 && rbdata->rectangle.height > 0)
@@ -995,7 +1108,7 @@ region_filter_func (GdkXEvent *xevent, GdkEvent *event, RbData *rbdata)
       break;
 
       /* The user is moving the mouse */
-      case XI_Motion:
+      case XI_Motion: /* MotionNotify */
         if (rbdata->pressed)
           {
             TRACE ("Mouse is moving");
@@ -1014,9 +1127,18 @@ region_filter_func (GdkXEvent *xevent, GdkEvent *event, RbData *rbdata)
                                 (unsigned int) rbdata->rectangle.height-1);
               }
 
-            device_event = (XIDeviceEvent*) x_event->xcookie.data;
-            x2 = device_event->root_x;
-            y2 = device_event->root_y;
+            if (is_xinput)
+              {
+                device_event = (XIDeviceEvent*) x_event->xcookie.data;
+                x2 = device_event->root_x;
+                y2 = device_event->root_y;
+              }
+            else
+              {
+                x2 = x_event->xkey.x_root;
+                y2 = x_event->xkey.y_root;
+              }
+            
 
             if (rbdata->move_rectangle)
               {
@@ -1063,9 +1185,16 @@ region_filter_func (GdkXEvent *xevent, GdkEvent *event, RbData *rbdata)
         return GDK_FILTER_REMOVE;
         break;
 
-      case XI_KeyPress:
-        device_event = (XIDeviceEvent*) x_event->xcookie.data;
-        key = device_event->detail;
+      case XI_KeyPress: /* KeyPress */
+        if (is_xinput)
+          {
+            device_event = (XIDeviceEvent*) x_event->xcookie.data;
+            key = device_event->detail;
+          }
+        else
+          {
+            key = x_event->xkey.keycode;
+          }
 
         if (rbdata->pressed && (
             key == XKeysymToKeycode (gdk_x11_get_default_xdisplay (), XK_Control_L) ||
@@ -1103,9 +1232,16 @@ region_filter_func (GdkXEvent *xevent, GdkEvent *event, RbData *rbdata)
           }
         break;
 
-      case XI_KeyRelease:
-        device_event = (XIDeviceEvent*) x_event->xcookie.data;
-        key = device_event->detail;
+      case XI_KeyRelease: /* KeyRelease */
+        if (is_xinput)
+          {
+            device_event = (XIDeviceEvent*) x_event->xcookie.data;
+            key = device_event->detail;
+          }
+        else
+          {
+            key = x_event->xkey.keycode;
+          }
 
         if (rbdata->pressed && (
             key == XKeysymToKeycode (gdk_x11_get_default_xdisplay (), XK_Control_L) ||
@@ -1140,7 +1276,6 @@ static GdkPixbuf
   gint screen;
   RbData rbdata;
   GdkCursor *xhair_cursor;
-  GdkDevice *pointer, *keyboard;
   GdkGrabStatus res;
   GdkSeat   *seat;
   long value_mask;
@@ -1160,37 +1295,19 @@ static GdkPixbuf
    * because xfsettings will grab the key for a moment */ 
   g_usleep(100000);
 
+  gdk_window_show_unraised (root_window);
+
   /* Grab the mouse and the keyboard to prevent any interaction with other
    * applications */
   seat = gdk_display_get_default_seat (gdk_display_get_default ());
-  pointer = gdk_seat_get_pointer (seat);
-  keyboard = gdk_seat_get_keyboard (seat);
 
-  res = gdk_device_grab (keyboard, root_window,
-                         GDK_OWNERSHIP_NONE, FALSE,
-                         GDK_KEY_PRESS_MASK |
-                         GDK_KEY_RELEASE_MASK,
-                         NULL, GDK_CURRENT_TIME);
+  res = gdk_seat_grab (seat, root_window, GDK_SEAT_CAPABILITY_ALL, FALSE,
+                       xhair_cursor, NULL, NULL, NULL);
 
   if (res != GDK_GRAB_SUCCESS)
     {
       g_object_unref (xhair_cursor);
-      g_warning ("Failed to grab keyboard");
-      return NULL;
-    }
-
-  res = gdk_device_grab (pointer, root_window,
-                         GDK_OWNERSHIP_NONE, FALSE,
-                         GDK_POINTER_MOTION_MASK |
-                         GDK_BUTTON_PRESS_MASK | 
-                         GDK_BUTTON_RELEASE_MASK,
-                         xhair_cursor, GDK_CURRENT_TIME);
-
-  if (res != GDK_GRAB_SUCCESS)
-    {
-      gdk_device_ungrab (keyboard, GDK_CURRENT_TIME);
-      g_object_unref (xhair_cursor);
-      g_warning ("Failed to grab pointer");
+      g_warning ("Failed to grab seat");
       return NULL;
     }
 
@@ -1227,7 +1344,7 @@ static GdkPixbuf
   gdk_window_add_filter (root_window,
                          (GdkFilterFunc) region_filter_func, &rbdata);
 
-  gdk_flush ();
+  gdk_display_flush (gdk_display_get_default ());
 
   gtk_main ();
 
@@ -1235,8 +1352,7 @@ static GdkPixbuf
                             (GdkFilterFunc) region_filter_func,
                             &rbdata);
 
-  gdk_device_ungrab (pointer, GDK_CURRENT_TIME);
-  gdk_device_ungrab (keyboard, GDK_CURRENT_TIME);
+  gdk_seat_ungrab (seat);
 
   /* Get the screenshot's pixbuf */
   if (G_LIKELY (!rbdata.cancelled))
@@ -1265,18 +1381,18 @@ static GdkPixbuf
 
 
 /**
- * screenshooter_take_screenshot:
+ * screenshooter_capture_screenshot:
  * @region: the region to be screenshoted. It can be FULLSCREEN,
  *          ACTIVE_WINDOW or SELECT.
- * @delay: the delay before the screenshot is taken, in seconds.
+ * @delay: the delay before the screenshot is captured, in seconds.
  * @mouse: whether the mouse pointer should be displayed on the screenshot.
  *
- * Takes a screenshot with the given options. If @region is FULLSCREEN,
- * the screenshot is taken after @delay seconds. If @region is
+ * Captures a screenshot with the given options. If @region is FULLSCREEN,
+ * the screenshot is captured after @delay seconds. If @region is
  * ACTIVE_WINDOW, a delay of @delay seconds elapses, then the active
  * window is detected and captured. If @region is SELECT, the user will
  * have to select a portion of the screen with the mouse. Then a delay of
- * @delay seconds elapses, and a screenshot is taken.
+ * @delay seconds elapses, and a screenshot is captured.
  *
  * @show_mouse is only taken into account when @region is FULLSCREEN
  * or ACTIVE_WINDOW.
@@ -1284,10 +1400,10 @@ static GdkPixbuf
  * Return value: a #GdkPixbuf containing the screenshot or %NULL
  * (if @region is SELECT, the user can cancel the operation).
  **/
-GdkPixbuf *screenshooter_take_screenshot (gint     region,
-                                          gint     delay,
-                                          gboolean show_mouse,
-                                          gboolean plugin)
+GdkPixbuf *screenshooter_capture_screenshot (gint     region,
+                                             gint     delay,
+                                             gboolean show_mouse,
+                                             gboolean plugin)
 {
   GdkPixbuf *screenshot = NULL;
   GdkWindow *window = NULL;
@@ -1300,7 +1416,7 @@ GdkPixbuf *screenshooter_take_screenshot (gint     region,
    * window has been grabbed. */
   gboolean needs_unref = TRUE;
 
-  /* Get the screen on which the screenshot should be taken */
+  /* Get the screen on which the screenshot should be captured */
   screen = gdk_screen_get_default ();
 
   /* Sync the display */
