@@ -137,16 +137,23 @@ screenshooter_write_rc_file (const gchar *file, ScreenshotData *sd)
 
   TRACE ("Write the entries.");
 
-  xfce_rc_write_int_entry (rc, "delay", sd->delay);
-  xfce_rc_write_int_entry (rc, "region", sd->region);
-  xfce_rc_write_int_entry (rc, "show_mouse", sd->show_mouse);
-  xfce_rc_write_entry (rc, "screenshot_dir", sd->screenshot_dir);
   xfce_rc_write_entry (rc, "app", sd->app);
   xfce_rc_write_entry (rc, "last_user", sd->last_user);
+  xfce_rc_write_entry (rc, "screenshot_dir", sd->screenshot_dir);
 
-  /* do not save if the action was specified from cli */
+  /* do not save if action was specified from cli */
   if (!sd->action_specified)
+  {
     xfce_rc_write_int_entry (rc, "action", sd->action);
+  }
+
+  /* do not save if region was specified from cli */
+  if (!sd->region_specified)
+  {
+    xfce_rc_write_int_entry (rc, "delay", sd->delay);
+    xfce_rc_write_int_entry (rc, "region", sd->region);
+    xfce_rc_write_int_entry (rc, "show_mouse", sd->show_mouse);
+  }
 
   TRACE ("Flush and close the rc file");
   xfce_rc_close (rc);
